@@ -1,6 +1,6 @@
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 ### BEGIN LICENSE
-# Peter Levi <peterlevi@peterlevi.com>
+# Copyright (c) 2012, Peter Levi <peterlevi@peterlevi.com>
 # This program is free software: you can redistribute it and/or modify it 
 # under the terms of the GNU General Public License version 3, as published 
 # by the Free Software Foundation.
@@ -24,6 +24,7 @@ from gi.repository import Gio, Gtk, Gdk, GObject, GdkPixbuf # pylint: disable=E0
 
 import threading
 from variety.Util import Util
+from variety_lib import varietyconfig
 from variety_lib.varietyconfig import get_data_file
 
 from variety.Options import Options
@@ -79,6 +80,28 @@ class PreferencesVarietyDialog(PreferencesDialog):
         self.loading = False
         self.reload()
 
+    def update_status_message(self):
+        msg = ""
+        if self.parent.server_options:
+            try:
+                msg_dict = self.parent.server_options.get("status_message", {})
+                ver = varietyconfig.get_version()
+                if ver in msg_dict:
+                    msg = msg_dict[ver].strip()
+                elif "*" in msg_dict:
+                    msg = msg_dict["*"].strip()
+            except Exception:
+                logger.exception("Could not parse status message")
+
+        logger.info("Showing status message: %s" % msg)
+        self.set_status_message(msg)
+
+    def set_status_message(self, msg):
+        def _update_ui():
+            self.ui.status_message.set_visible(msg)
+            self.ui.status_message.set_markup(msg)
+        GObject.idle_add(_update_ui)
+
     def reload(self):
         try:
             logger.info("Reloading preferences dialog")
@@ -98,16 +121,14 @@ class PreferencesVarietyDialog(PreferencesDialog):
             self.set_download_interval(self.options.download_interval)
 
             self.ui.download_folder_chooser.set_filename(os.path.expanduser(self.options.download_folder))
-            self.ui.download_folder_chooser.set_current_folder(os.path.expanduser(self.options.download_folder))
+            self.update_real_download_folder()
 
             self.ui.quota_enabled.set_active(self.options.quota_enabled)
             self.ui.quota_size.set_text(str(self.options.quota_size))
 
             self.ui.favorites_folder_chooser.set_filename(os.path.expanduser(self.options.favorites_folder))
-            self.ui.favorites_folder_chooser.set_current_folder(os.path.expanduser(self.options.favorites_folder))
 
             self.ui.fetched_folder_chooser.set_filename(os.path.expanduser(self.options.fetched_folder))
-            self.ui.fetched_folder_chooser.set_current_folder(os.path.expanduser(self.options.fetched_folder))
             self.ui.clipboard_enabled.set_active(self.options.clipboard_enabled)
             self.ui.clipboard_use_whitelist.set_active(self.options.clipboard_use_whitelist)
             self.ui.clipboard_hosts.get_buffer().set_text('\n'.join(self.options.clipboard_hosts))
@@ -226,6 +247,7 @@ class PreferencesVarietyDialog(PreferencesDialog):
 
             self.build_add_button_menu()
 
+            self.update_status_message()
             self.dialog = None
         finally:
             # To be sure we are completely loaded, pass via two hops: first delay, then idle_add:
@@ -244,16 +266,24 @@ class PreferencesVarietyDialog(PreferencesDialog):
 
         self.add_menu.popup(None, self.ui.add_button, position, None, 0, Gtk.get_current_event_time())
 
+    def on_remove_sources_clicked(self, widget=None):
+        def position(x, y):
+            button_alloc = self.ui.remove_sources.get_allocation()
+            window_pos = self.ui.remove_sources.get_window().get_position()
+            return button_alloc.x + window_pos[0], button_alloc.y + button_alloc.height + window_pos[1], True
+
+        self.build_remove_button_menu().popup(None, self.ui.remove_sources, position, None, 0, Gtk.get_current_event_time())
+
     def build_add_button_menu(self):
         self.add_menu = Gtk.Menu()
 
         items = [
-            ("Images", self.on_add_images_clicked),
-            ("Folders", self.on_add_folders_clicked),
-            ("Flickr", self.on_add_flickr_clicked),
-            ("Wallbase.cc", self.on_add_wallbase_clicked),
-            ("Wallpapers.net", self.on_add_wn_clicked),
-            ("Media RSS", self.on_add_mediarss_clicked),
+            (_("Images"), self.on_add_images_clicked),
+            (_("Folders"), self.on_add_folders_clicked),
+            (_("Flickr"), self.on_add_flickr_clicked),
+            (_("Wallbase.cc"), self.on_add_wallbase_clicked),
+            (_("Wallpapers.net"), self.on_add_wn_clicked),
+            (_("Media RSS"), self.on_add_mediarss_clicked),
         ]
 
         for x in items:
@@ -264,6 +294,34 @@ class PreferencesVarietyDialog(PreferencesDialog):
 
         self.add_menu.show_all()
 
+    def build_remove_button_menu(self):
+        model, rows = self.ui.sources.get_selection().get_selected_rows()
+
+        has_downloaders = False
+        for row in rows:
+            type = Options.str_to_type(model[row][1])
+            if type in Options.SourceType.dl_types and type not in UNREMOVEABLE_TYPES:
+                has_downloaders = True
+
+        remove_menu = Gtk.Menu()
+        item1 = Gtk.MenuItem()
+        item1.set_label(_("Remove the source, keep the files") if len(rows) == 1 else
+            _("Remove the sources, keep the files"))
+        item1.connect("activate", self.remove_sources)
+        remove_menu.append(item1)
+
+        item2 = Gtk.MenuItem()
+        def _remove_with_files(widget=None):
+            self.remove_sources(delete_files=True)
+        item2.set_label(_("Remove the source and delete the downloaded files") if len(rows) == 1 else
+            _("Remove the sources and delete the downloaded files"))
+        item2.connect("activate", _remove_with_files)
+        item2.set_sensitive(has_downloaders)
+        remove_menu.append(item2)
+
+        remove_menu.show_all()
+        return remove_menu
+
     def source_enabled_toggled(self, widget, path, model):
         row = model[path]
         row[0] = not row[0]
@@ -293,10 +351,9 @@ class PreferencesVarietyDialog(PreferencesDialog):
                     _("Using the World Sunlight Map requires both downloading and changing "
                     "enabled at intervals of 30 minutes or less. Settings were adjusted automatically."))
 
-    def set_time(self, interval, text, time_unit):
+    def set_time(self, interval, text, time_unit, times=(1, 60, 60 * 60, 24 * 60 * 60)):
         if interval < 5:
             interval = 5
-        times = [1, 60, 60 * 60, 24 * 60 * 60]
         x = len(times) - 1
         while times[x] > interval:
             x -= 1
@@ -308,7 +365,8 @@ class PreferencesVarietyDialog(PreferencesDialog):
         self.set_time(seconds, self.ui.change_interval_text, self.ui.change_interval_time_unit)
 
     def set_download_interval(self, seconds):
-        self.set_time(seconds, self.ui.download_interval_text, self.ui.download_interval_time_unit)
+        self.set_time(seconds, self.ui.download_interval_text, self.ui.download_interval_time_unit,
+                      times=(60, 60 * 60, 24 * 60 * 60))
 
     def set_quotes_change_interval(self, seconds):
         self.set_time(seconds, self.ui.quotes_change_interval_text, self.ui.quotes_change_interval_time_unit)
@@ -334,7 +392,7 @@ class PreferencesVarietyDialog(PreferencesDialog):
 
     def get_download_interval(self):
         return self.read_time(
-            self.ui.download_interval_text, self.ui.download_interval_time_unit, 30, self.options.download_interval)
+            self.ui.download_interval_text, self.ui.download_interval_time_unit, 60, self.options.download_interval)
 
     def get_quotes_change_interval(self):
         return self.read_time(
@@ -429,8 +487,16 @@ class PreferencesVarietyDialog(PreferencesDialog):
                 self.ui.sources.scroll_to_cell(i, None, False, 0, 0)
                 return
 
-    def on_remove_sources_clicked(self, widget=None):
+    def remove_sources(self, widget=None, delete_files = False):
         model, rows = self.ui.sources.get_selection().get_selected_rows()
+
+        if delete_files:
+            for row in rows:
+                type = Options.str_to_type(model[row][1])
+                if type in Options.SourceType.dl_types and type not in UNREMOVEABLE_TYPES:
+                    source = self.model_row_to_source(model[row])
+                    self.parent.delete_files_of_source(source)
+
         # store the treeiters from paths
         iters = []
         for row in rows:
@@ -527,6 +593,9 @@ class PreferencesVarietyDialog(PreferencesDialog):
 
         self.ui.remove_sources.set_sensitive(len(rows) > 0)
 
+    def model_row_to_source(self, row):
+        return [row[0], Options.str_to_type(row[1]), row[2]]
+
     def show_thumbs(self, sources):
         try:
             if not sources:
@@ -547,7 +616,7 @@ class PreferencesVarietyDialog(PreferencesDialog):
                     image_count += 1
                     images.append(source[2])
                 else:
-                    folder = self.parent.get_folder_of_source([source[0], Options.str_to_type(source[1]), source[2]])
+                    folder = self.parent.get_folder_of_source(self.model_row_to_source(source))
                     image_count += sum(1 for f in Util.list_files(folders=(folder,), filter_func=Util.is_image, max_files=1, randomize=False))
                     folders.append(folder)
 
@@ -614,6 +683,7 @@ class PreferencesVarietyDialog(PreferencesDialog):
 
     def close(self):
         self.hide()
+        self.parent.trigger_download()
         self.on_destroy()
 
     def on_save_clicked(self, widget):
@@ -847,7 +917,6 @@ class PreferencesVarietyDialog(PreferencesDialog):
     def on_quota_enabled_toggled(self, widget = None):
         active = self.ui.download_enabled.get_active() and self.ui.quota_enabled.get_active()
         self.ui.quota_size.set_sensitive(active)
-        self.ui.quota_label.set_sensitive(active)
 
     def on_desired_color_enabled_toggled(self, widget = None):
         self.ui.desired_color.set_sensitive(self.ui.desired_color_enabled.get_active())
@@ -872,7 +941,6 @@ class PreferencesVarietyDialog(PreferencesDialog):
             except Exception:
                 pass
         self.parent.thumbs_manager.hide(gdk_thread=True, force=False)
-        self.parent.trigger_download()
 
     def on_downloaded_changed(self, widget=None):
         if not os.access(self.ui.download_folder_chooser.get_filename(), os.W_OK):
@@ -882,9 +950,15 @@ class PreferencesVarietyDialog(PreferencesDialog):
 
         if not self.loading and self.ui.quota_enabled.get_active():
             self.ui.quota_enabled.set_active(False)
-            self.parent.show_notification("Limit disabled",
-                                          "Changing the download folder automatically turns off the size limit "
-                                          "to prevent from accidental data loss")
+            self.parent.show_notification(
+                _("Limit disabled"),
+                _("Changing the download folder automatically turns off the size limit to prevent from accidental data loss"),
+                important=True)
+
+    def update_real_download_folder(self):
+        if not Util.same_file_paths(self.parent.options.download_folder, self.parent.real_download_folder):
+            self.ui.real_download_folder.set_visible(True)
+        self.ui.real_download_folder.set_text(_("Actual download folder: %s ") % self.parent.real_download_folder)
 
     def on_favorites_changed(self, widget=None):
         if not os.access(self.ui.favorites_folder_chooser.get_filename(), os.W_OK):
