@@ -1,6 +1,6 @@
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 ### BEGIN LICENSE
-# Peter Levi <peterlevi@peterlevi.com>
+# Copyright (c) 2012, Peter Levi <peterlevi@peterlevi.com>
 # This program is free software: you can redistribute it and/or modify it 
 # under the terms of the GNU General Public License version 3, as published 
 # by the Free Software Foundation.
@@ -20,7 +20,6 @@ import subprocess
 import urllib
 from variety.VarietyOptionParser import VarietyOptionParser
 from variety.FacebookHelper import FacebookHelper
-from variety_lib.varietyconfig import get_version
 
 gettext.textdomain('variety')
 
@@ -63,12 +62,21 @@ from variety.QuoteWriter import QuoteWriter
 from variety import indicator
 
 
+DL_FOLDER_FILE = ".variety_download_folder"
+
 class VarietyWindow(Gtk.Window):
     __gtype_name__ = "VarietyWindow"
 
+    SERVERSIDE_OPTIONS_URL = "http://smarturl.it/varietyserveroptions"
     MAX_FILES = 10000
-    SCHEMA = 'org.gnome.desktop.background'
-    KEY = 'picture-uri'
+
+    OUTDATED_SET_WP_SCRIPTS = {
+        "b8ff9cb65e3bb7375c4e2a6e9611c7f8",
+        "3729d3e1f57aa1159988ba2c8f929389",
+        "feafa658d9686ecfabdbcf236c32fd0f",
+        "83d8ebeec3676474bdd90c55417e8640",
+        "1562cb289319aa39ac1b37a8ee4c0103"
+    }
 
     def __init__(self):
         pass
@@ -81,7 +89,7 @@ class VarietyWindow(Gtk.Window):
 
         self.ind = None
 
-        self.gsettings = Gio.Settings.new(self.SCHEMA)
+        self.gsettings = Gio.Settings.new('org.gnome.desktop.background')
 
         self.thumbs_manager = ThumbsManager(self)
 
@@ -111,10 +119,14 @@ class VarietyWindow(Gtk.Window):
 
         # load config
         self.options = None
+        self.server_options = {}
         self.load_banned()
         self.load_history()
-        self.thumbs_manager.mark_active(file=self.used[self.position], position=self.position)
-        self.reload_config()
+
+        if self.position < len(self.used):
+            self.thumbs_manager.mark_active(file=self.used[self.position], position=self.position)
+
+        self.reload_config(initial_run=True)
         self.load_last_change_time()
 
         self.image_count = -1
@@ -191,7 +203,6 @@ class VarietyWindow(Gtk.Window):
 
     def prepare_config_folder(self):
         self.config_folder = os.path.expanduser("~/.config/variety")
-
         Util.makedirs(self.config_folder)
 
         shutil.copy(varietyconfig.get_data_file("config", "variety.conf"),
@@ -207,11 +218,29 @@ class VarietyWindow(Gtk.Window):
                         varietyconfig.get_data_file("config", "ui.conf"))
             shutil.copy(varietyconfig.get_data_file("config", "ui.conf"), self.config_folder)
 
+        self.scripts_folder = os.path.join(self.config_folder, "scripts")
+        Util.makedirs(self.scripts_folder)
+
+        if not os.path.exists(os.path.join(self.scripts_folder, "set_wallpaper")):
+            logger.info("Missing set_wallpaper file, copying it from " +
+                        varietyconfig.get_data_file("scripts", "set_wallpaper"))
+            shutil.copy(varietyconfig.get_data_file("scripts", "set_wallpaper"), self.scripts_folder)
+
+        if not os.path.exists(os.path.join(self.scripts_folder, "get_wallpaper")):
+            logger.info("Missing get_wallpaper file, copying it from " +
+                        varietyconfig.get_data_file("scripts", "get_wallpaper"))
+            shutil.copy(varietyconfig.get_data_file("scripts", "get_wallpaper"), self.scripts_folder)
+
+        # make all scripts executable:
+        for f in os.listdir(self.scripts_folder):
+            path = os.path.join(self.scripts_folder, f)
+            os.chmod(path, os.stat(path).st_mode | stat.S_IEXEC)
+
         # TODO: Sort of hacky to have filter-related code here, they should be more isolated
         pencil_tile_filename = os.path.join(self.config_folder, "pencil_tile.png")
         if not os.path.exists(pencil_tile_filename):
             def _generate_pencil_tile():
-                logger.info("Missing pencil_tile.png file, generating it" +
+                logger.info("Missing pencil_tile.png file, generating it " +
                             varietyconfig.get_data_file("media", "pencil_tile.png"))
                 try:
                     os.system(
@@ -221,16 +250,6 @@ class VarietyWindow(Gtk.Window):
                     logger.exception("Could not generate pencil_tile.gif")
             threading.Timer(0, _generate_pencil_tile).start()
 
-        self.scripts_folder = os.path.join(self.config_folder, "scripts")
-        if not os.path.exists(self.scripts_folder):
-            logger.info("Missing scripts dir, copying it from " + varietyconfig.get_data_file("scripts"))
-            shutil.copytree(varietyconfig.get_data_file("scripts"), self.scripts_folder)
-
-        # make all scripts executable:
-        for f in os.listdir(self.scripts_folder):
-            path = os.path.join(self.scripts_folder, f)
-            os.chmod(path, os.stat(path).st_mode | stat.S_IEXEC)
-
     def register_clipboard(self):
         def clipboard_changed(clipboard, event):
             try:
@@ -260,15 +279,45 @@ class VarietyWindow(Gtk.Window):
             logger.info("%s = %s" % (k, v))
 #        pprint(self.options.__dict__, indent=0)
 
-    def reload_config(self):
+    def get_real_download_folder(self):
+        subfolder = "Downloaded by Variety"
+        dl = self.options.download_folder
+
+        # If chosen folder is within Variety's config folder, or folder's name is "Downloaded by Variety",
+        # or folder is missing or it is empty or it has already been used as a download folder, then use it:
+        if Util.file_in(dl, self.config_folder) or \
+                dl.endswith("/%s" % subfolder) or dl.endswith("/%s/" % subfolder) or \
+                not os.path.exists(dl) or not os.listdir(dl) or \
+                os.path.exists(os.path.join(dl, DL_FOLDER_FILE)):
+            return dl
+        else:
+            # In all other cases (i.e. it is an existing user folder with files in it), use a subfolder inside it
+            return os.path.join(dl, subfolder)
+
+    def prepare_download_folder(self):
+        self.real_download_folder = self.get_real_download_folder()
+        if self.preferences_dialog:
+            GObject.idle_add(self.preferences_dialog.update_real_download_folder)
+
+        Util.makedirs(self.real_download_folder)
+        dl_folder_file = os.path.join(self.real_download_folder, DL_FOLDER_FILE)
+        if not os.path.exists(dl_folder_file):
+            with open(dl_folder_file, "w") as f:
+                f.write(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()))
+
+    def reload_config(self, initial_run=False):
         self.previous_options = self.options
 
         self.options = Options()
         self.options.read()
 
+        if initial_run:
+            self.perform_upgrade_after_loading_options()
+
         GObject.idle_add(self.update_indicator_icon)
 
-        Util.makedirs(self.options.download_folder)
+        self.prepare_download_folder()
+
         Util.makedirs(self.options.favorites_folder)
         Util.makedirs(self.options.fetched_folder)
 
@@ -332,6 +381,8 @@ class VarietyWindow(Gtk.Window):
             with self.prepared_lock:
                 self.prepared_cleared = True
                 self.prepared = []
+                self.prepared_from_downloads = []
+                self.prepare_event.set()
             self.image_count = -1
         else:
             logger.info("No need to clear prepared queue")
@@ -427,6 +478,25 @@ class VarietyWindow(Gtk.Window):
             dlr.update_download_folder()
             return dlr.target_folder
 
+    def delete_files_of_source(self, source):
+        folder = self.get_folder_of_source(source)
+        if Util.file_in(folder, self.real_download_folder):
+            self.remove_folder_from_queues(folder)
+            should_repaint = \
+                self.thumbs_manager.is_showing("history") or self.thumbs_manager.is_showing("downloads") or (
+                self.thumbs_manager.get_folders() is not None and folder in self.thumbs_manager.get_folders())
+
+            if should_repaint:
+                self.thumbs_manager.repaint()
+            try:
+                logger.info("Deleting recursively folder " + folder)
+                shutil.rmtree(folder)
+            except Exception:
+                logger.exception("Could not delete download folder contents " + folder)
+            if Util.file_in(self.current, folder):
+                change_timer = threading.Timer(1, self.next_wallpaper)
+                change_timer.start()
+
     def load_banned(self):
         self.banned = set()
         try:
@@ -462,6 +532,10 @@ class VarietyWindow(Gtk.Window):
 
         self.events.extend([self.change_event, self.prepare_event, self.dl_event])
 
+        server_options_thread = threading.Thread(target=self.server_options_thread)
+        server_options_thread.daemon = True
+        server_options_thread.start()
+
     def is_in_favorites(self, file):
         filename = os.path.basename(file)
         return os.path.exists(os.path.join(self.options.favorites_folder, filename))
@@ -524,62 +598,64 @@ class VarietyWindow(Gtk.Window):
 
             deleteable = os.access(file, os.W_OK) and not self.is_current_refreshable()
             favs_op = self.determine_favorites_operation(file)
+            image_source = self.get_source(file)
 
             if not is_gtk_thread:
                 Gdk.threads_enter()
 
-            rating_menu = None
-            if deleteable and self.options.show_rating_enabled:
-                rating_menu = ThumbsManager.create_rating_menu(file, self)
+            try:
+                rating_menu = None
+                if deleteable and self.options.show_rating_enabled:
+                    rating_menu = ThumbsManager.create_rating_menu(file, self)
 
-            for i in xrange(10):
-                self.ind.prev.set_sensitive(self.position < len(self.used) - 1)
-                self.ind.file_label.set_label(os.path.basename(file).replace('_', '__'))
+                for i in xrange(5):    # if only done once, the menu is not always updated for some reason
+                    self.ind.prev.set_sensitive(self.position < len(self.used) - 1)
+                    self.ind.file_label.set_label(os.path.basename(file).replace('_', '__'))
 
-                self.ind.focus.set_sensitive(self.get_source(file) is not None)
+                    self.ind.focus.set_sensitive(image_source is not None)
 
-                # delay enabling Trash if auto_changed
-                self.ind.trash.set_sensitive(deleteable and not auto_changed)
+                    # delay enabling Trash if auto_changed
+                    self.ind.trash.set_sensitive(deleteable and not auto_changed)
 
-                self.update_favorites_menuitems(self.ind, auto_changed, favs_op)
+                    self.update_favorites_menuitems(self.ind, auto_changed, favs_op)
 
-                self.ind.show_origin.set_label(label)
-                self.ind.show_origin.set_sensitive(True)
+                    self.ind.show_origin.set_label(label)
+                    self.ind.show_origin.set_sensitive(True)
 
-                self.ind.rating_separator.set_visible(self.options.show_rating_enabled)
-                self.ind.rating.set_visible(self.options.show_rating_enabled)
-                self.ind.rating.set_sensitive(rating_menu is not None)
-                if rating_menu:
-                    self.ind.rating.set_submenu(rating_menu)
+                    self.ind.rating_separator.set_visible(self.options.show_rating_enabled)
+                    self.ind.rating.set_visible(self.options.show_rating_enabled)
+                    self.ind.rating.set_sensitive(rating_menu is not None)
+                    if rating_menu:
+                        self.ind.rating.set_submenu(rating_menu)
 
-                self.ind.history.handler_block(self.ind.history_handler_id)
-                self.ind.history.set_active(self.thumbs_manager.is_showing("history"))
-                self.ind.history.handler_unblock(self.ind.history_handler_id)
+                    self.ind.history.handler_block(self.ind.history_handler_id)
+                    self.ind.history.set_active(self.thumbs_manager.is_showing("history"))
+                    self.ind.history.handler_unblock(self.ind.history_handler_id)
 
-                self.ind.downloads.set_visible(self.options.download_enabled)
-                self.ind.downloads.set_sensitive(len(self.downloaded) > 0)
-                self.ind.downloads.handler_block(self.ind.downloads_handler_id)
-                self.ind.downloads.set_active(self.thumbs_manager.is_showing("downloads"))
-                self.ind.downloads.handler_unblock(self.ind.downloads_handler_id)
+                    self.ind.downloads.set_visible(self.options.download_enabled)
+                    self.ind.downloads.set_sensitive(len(self.downloaded) > 0)
+                    self.ind.downloads.handler_block(self.ind.downloads_handler_id)
+                    self.ind.downloads.set_active(self.thumbs_manager.is_showing("downloads"))
+                    self.ind.downloads.handler_unblock(self.ind.downloads_handler_id)
 
-                self.ind.publish_fb.set_visible(self.options.facebook_enabled)
-                self.ind.publish_fb.set_sensitive(self.url is not None)
+                    self.ind.publish_fb.set_visible(self.options.facebook_enabled)
+                    self.ind.publish_fb.set_sensitive(self.url is not None)
 
-                self.ind.pause_resume.set_label(_("Pause") if self.options.change_enabled else _("Resume"))
+                    self.ind.pause_resume.set_label(_("Pause") if self.options.change_enabled else _("Resume"))
 
-                self.ind.quotes.set_visible(self.options.quotes_enabled and self.quote is not None)
-                if self.quotes_engine:
-                    self.ind.prev_quote.set_sensitive(self.quotes_engine.has_previous())
-                self.ind.quote_clipboard.set_sensitive(self.options.quotes_enabled and self.quote is not None)
-                self.ind.quotes_pause_resume.set_label(_("Pause") if self.options.quotes_change_enabled else _("Resume"))
-
-            if not is_gtk_thread:
-                Gdk.threads_leave()
+                    self.ind.quotes.set_visible(self.options.quotes_enabled and self.quote is not None)
+                    if self.quotes_engine:
+                        self.ind.prev_quote.set_sensitive(self.quotes_engine.has_previous())
+                    self.ind.quote_clipboard.set_sensitive(self.options.quotes_enabled and self.quote is not None)
+                    self.ind.quotes_pause_resume.set_label(_("Pause") if self.options.quotes_change_enabled else _("Resume"))
+            finally:
+                if not is_gtk_thread:
+                    Gdk.threads_leave()
 
             # delay enabling Move/Copy operations after automatic changes - protect from inadvertent clicks
             if auto_changed:
                 def update_file_operations():
-                    for i in xrange(10):
+                    for i in xrange(5):
                         self.ind.trash.set_sensitive(deleteable)
                         self.ind.copy_to_favorites.set_sensitive(favs_op in ("copy", "both"))
                         self.ind.move_to_favorites.set_sensitive(favs_op in ("move", "both"))
@@ -647,6 +723,7 @@ class VarietyWindow(Gtk.Window):
                 logger.exception("Exception in clock_thread")
 
     def find_images(self):
+        self.prepared_cleared = False
         images = self.select_random_images(100)
 
         found = set()
@@ -654,16 +731,15 @@ class VarietyWindow(Gtk.Window):
             if len(found) > 10 or len(found) >= len(images):
                 break
             for img in images:
-                if not self.running:
-                    return
-                if self.prepared_cleared:
-                    self.prepared_cleared = False
+                if not self.running or self.prepared_cleared:
+                    # abandon this search
                     return
+
                 try:
                     if not img in found and self.image_ok(img, fuzziness):
                         #print "OK at fz %d: %s" % (fuzziness, img)
                         found.add(img)
-                        if len(self.prepared) < 3:
+                        if len(self.prepared) < 3 and not self.prepared_cleared:
                             with self.prepared_lock:
                                 self.prepared.append(img)
                 except Exception:
@@ -671,8 +747,9 @@ class VarietyWindow(Gtk.Window):
 
         with self.prepared_lock:
             if self.prepared_cleared:
-                self.prepared_cleared = False
+                # abandon this search
                 return
+
             self.prepared.extend(found)
             if not self.prepared and images:
                 logger.info("Prepared buffer still empty after search, appending some non-ok image")
@@ -694,6 +771,7 @@ class VarietyWindow(Gtk.Window):
                     _("Variety is finding too few images that match your image filtering criteria"))
 
     def prepare_thread(self):
+        time.sleep(20)
         logger.info("Prepare thread running")
         while self.running:
             try:
@@ -710,6 +788,26 @@ class VarietyWindow(Gtk.Window):
             self.prepare_event.wait()
             self.prepare_event.clear()
 
+    def server_options_thread(self):
+        time.sleep(20)
+        attempts = 0
+        while self.running:
+            try:
+                attempts += 1
+                logger.info("Fetching server options from %s" % VarietyWindow.SERVERSIDE_OPTIONS_URL)
+                self.server_options = Util.fetch_json(VarietyWindow.SERVERSIDE_OPTIONS_URL)
+                logger.info("Fetched server options: %s" % str(self.server_options))
+                if self.preferences_dialog:
+                    self.preferences_dialog.update_status_message()
+            except Exception:
+                logger.exception("Could not fetch Variety serverside options")
+                if attempts < 5:
+                    # the first several attempts may easily fail if Variety is run on startup, try again soon:
+                    time.sleep(30)
+                    continue
+
+            time.sleep(3600 * 24) # Update once daily
+
     def has_real_downloaders(self):
         return sum(1 for d in self.downloaders if not d.is_refresher) > 0
 
@@ -785,15 +883,15 @@ class VarietyWindow(Gtk.Window):
             return
 
         if self.download_folder_size <= 0 or random.randint(0, 20) == 0:
-            self.download_folder_size = self.get_folder_size(self.options.download_folder)
+            self.download_folder_size = self.get_folder_size(self.real_download_folder)
             logger.info("Refreshed download folder size: %d mb", self.download_folder_size / (1024.0 * 1024.0))
 
         mb_quota = self.options.quota_size * 1024 * 1024
         if self.download_folder_size > 0.95 * mb_quota:
-            logger.info("Purging oldest files from download folder, current size: %d mb" %
-                        int(self.download_folder_size / (1024.0 * 1024.0)))
+            logger.info("Purging oldest files from download folder %s, current size: %d mb" %
+                        (self.real_download_folder, int(self.download_folder_size / (1024.0 * 1024.0))))
             files = []
-            for dirpath, dirnames, filenames in os.walk(self.options.download_folder):
+            for dirpath, dirnames, filenames in os.walk(self.real_download_folder):
                 for f in filenames:
                     if Util.is_image(f):
                         fp = os.path.join(dirpath, f)
@@ -833,10 +931,11 @@ class VarietyWindow(Gtk.Window):
         CLOCK_ONLY = 3
 
     def set_wp_throttled(self, filename, delay=0.3, refresh_level=RefreshLevel.ALL):
-        self.thumbs_manager.mark_active(file=self.used[self.position], position=self.position)
+        self.thumbs_manager.mark_active(file=filename, position=self.position)
         if self.set_wp_timer:
             self.set_wp_timer.cancel()
-        def _do_set_wp(): self.do_set_wp(filename, refresh_level)
+        def _do_set_wp():
+            self.do_set_wp(filename, refresh_level)
         self.set_wp_timer = threading.Timer(delay, _do_set_wp)
         self.set_wp_timer.start()
 
@@ -844,14 +943,16 @@ class VarietyWindow(Gtk.Window):
         if not self.filters:
             return None
 
+        filter = random.choice(self.filters).strip()
+        if not filter:
+            return None
+
         w = Gdk.Screen.get_default().get_width()
         h = Gdk.Screen.get_default().get_height()
         cmd = 'convert "%s" -scale %dx%d^ ' % (filename, w, h)
 
-        filter = self.filters[random.randint(0, len(self.filters) - 1)]
-        if filter.strip():
-            logger.info("Applying filter: " + filter)
-            cmd += filter + ' '
+        logger.info("Applying filter: " + filter)
+        cmd += filter + ' '
 
         cmd = cmd + ' "' + os.path.join(self.config_folder, "wallpaper-filter.jpg") + '"'
         logger.info("ImageMagick filter cmd: " + cmd)
@@ -929,13 +1030,14 @@ class VarietyWindow(Gtk.Window):
                     or not hasattr(self, "post_filter_filename"):
                         self.post_filter_filename = to_set
                         cmd = self.build_imagemagick_filter_cmd(filename)
-                        result = os.system(cmd)
-                        if result == 0: #success
-                            to_set = os.path.join(self.config_folder, "wallpaper-filter.jpg")
-                            self.post_filter_filename = to_set
-                            self.write_filtered_wallpaper_origin(filename)
-                        else:
-                            logger.warning("Could not execute filter convert command - missing ImageMagick or bad filter defined?")
+                        if cmd:
+                            result = os.system(cmd)
+                            if result == 0: #success
+                                to_set = os.path.join(self.config_folder, "wallpaper-filter.jpg")
+                                self.post_filter_filename = to_set
+                                self.write_filtered_wallpaper_origin(filename)
+                            else:
+                                logger.warning("Could not execute filter convert command - missing ImageMagick or bad filter defined?")
                     else:
                         to_set = self.post_filter_filename
 
@@ -1053,15 +1155,22 @@ class VarietyWindow(Gtk.Window):
         else:
             logger.warning("Invalid position passed to move_to_history_position, %d, used len is %d" % (position, len(self.used)))
 
-    def show_notification(self, title, message="", icon=None):
+    def show_notification(self, title, message="", icon=None, important=False):
         if not icon:
             icon = varietyconfig.get_data_file("media", "variety.svg")
-        try:
-            self.notification.update(title, message, icon)
-        except AttributeError:
-            self.notification = Notify.Notification.new(title, message, icon)
-        self.notification.set_urgency(Notify.Urgency.LOW)
-        self.notification.show()
+
+        if not important:
+            try:
+                self.notification.update(title, message, icon)
+            except AttributeError:
+                self.notification = Notify.Notification.new(title, message, icon)
+            self.notification.set_urgency(Notify.Urgency.LOW)
+            self.notification.show()
+        else:
+            # use a separate notification that will not be updated with a non-important message
+            notification = Notify.Notification.new(title, message, icon)
+            notification.set_urgency(Notify.Urgency.NORMAL)
+            notification.show()
 
     def change_wallpaper(self, widget=None):
         try:
@@ -1140,11 +1249,13 @@ class VarietyWindow(Gtk.Window):
             self.update_indicator(auto_changed=False)
         GObject.idle_add(_update_indicator)
 
-        should_show = self.thumbs_manager.is_showing("downloads") or (self.thumbs_manager.get_folders() is not None \
-          and sum(1 for f in self.thumbs_manager.get_folders() if Util.file_in(added_image, f)) > 0)
+        should_show = self.thumbs_manager.is_showing("downloads") or (
+            self.thumbs_manager.get_folders() is not None \
+                and sum(1 for f in self.thumbs_manager.get_folders() if Util.file_in(added_image, f)) > 0)
 
         if should_show:
-            def _add(): self.thumbs_manager.add_image(added_image, gdk_thread=False)
+            def _add():
+                self.thumbs_manager.add_image(added_image, gdk_thread=False)
             add_timer = threading.Timer(0, _add)
             add_timer.start()
 
@@ -1338,11 +1449,19 @@ class VarietyWindow(Gtk.Window):
             logger.exception("Could not ban URL")
 
     def remove_from_queues(self, file):
+        self.position = max(0, self.position - sum(1 for f in self.used[:self.position] if f == file))
         self.used = [f for f in self.used if f != file]
         self.downloaded = [f for f in self.downloaded if f != file]
         with self.prepared_lock:
             self.prepared = [f for f in self.prepared if f != file]
 
+    def remove_folder_from_queues(self, folder):
+        self.position = max(0, self.position - sum(1 for f in self.used[:self.position] if Util.file_in(f, folder)))
+        self.used = [f for f in self.used if not Util.file_in(f, folder)]
+        self.downloaded = [f for f in self.downloaded if not Util.file_in(f, folder)]
+        with self.prepared_lock:
+            self.prepared = [f for f in self.prepared if not Util.file_in(f, folder)]
+
     def copy_to_favorites(self, widget=None, file=None):
         try:
             if not file:
@@ -1388,7 +1507,7 @@ class VarietyWindow(Gtk.Window):
             folder = pair[0]
             folder_lower = folder.lower().strip()
             if folder_lower == "downloaded":
-                folder = self.options.download_folder
+                folder = self.real_download_folder
             elif folder_lower == "fetched":
                 folder = self.options.fetched_folder
             elif folder_lower == "others":
@@ -1436,11 +1555,53 @@ class VarietyWindow(Gtk.Window):
     def first_run(self):
         fr_file = os.path.join(self.config_folder, ".firstrun")
         if not os.path.exists(fr_file):
-            f = open(fr_file, "w")
-            f.write(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()))
-            f.close()
+            with open(fr_file, "w") as f:
+                f.write(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()))
             self.show_welcome_dialog()
 
+    def write_current_version(self):
+        current_version = varietyconfig.get_version()
+        logger.info("Writing current version %s to .version" % current_version)
+        with open(os.path.join(self.config_folder, ".version"), "w") as f:
+            f.write(current_version)
+
+    def perform_upgrade_after_loading_options(self):
+        current_version = varietyconfig.get_version()
+
+        if not os.path.exists(os.path.join(self.config_folder, ".firstrun")):
+            # running for the first time
+            last_version = current_version
+            self.write_current_version()
+        else:
+            try:
+                with open(os.path.join(self.config_folder, ".version")) as f:
+                        last_version = f.read().strip()
+            except Exception:
+                last_version = "0.4.12" # this is the last release that did not have the .version file
+
+        logger.info("Last run version was %s or earlier, current version is %s" % (last_version, current_version))
+
+        if Util.compare_versions(last_version, "0.4.13") < 0:
+            logger.info("Performing upgrade to 0.4.13")
+
+            set_wallpaper_file = os.path.join(self.scripts_folder, "set_wallpaper")
+            if not os.path.exists(set_wallpaper_file) or\
+               Util.md5file(set_wallpaper_file) in VarietyWindow.OUTDATED_SET_WP_SCRIPTS:
+                logger.info("Outdated set_wallpaper file, copying it from " +
+                            varietyconfig.get_data_file("scripts", "set_wallpaper"))
+                shutil.copy(varietyconfig.get_data_file("scripts", "set_wallpaper"), self.scripts_folder)
+
+            logger.info("Writing %s to current download folder %s" % (DL_FOLDER_FILE, self.options.download_folder))
+            # mark the current download folder as a valid download folder
+            Util.makedirs(self.options.download_folder)
+            dl_folder_file = os.path.join(self.options.download_folder, DL_FOLDER_FILE)
+            if not os.path.exists(dl_folder_file):
+                with open(dl_folder_file, "w") as f:
+                    f.write(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()))
+
+        if Util.compare_versions(last_version, current_version) < 0:
+            self.write_current_version()
+
     def show_welcome_dialog(self):
         dialog = WelcomeDialog()
         if os.environ.get('KDE_FULL_SESSION') == 'true':
@@ -1491,7 +1652,7 @@ Passing local files will add them to Variety's queue.
 Passing remote URLs will make Variety fetch them to Fetched folder and place them in the queue.
 
 To set a specific wallpaper: %prog /some/local/image.jpg --next""")
-        parser = VarietyOptionParser(usage=usage, version="%%prog %s" % get_version(), report_errors=report_errors)
+        parser = VarietyOptionParser(usage=usage, version="%%prog %s" % varietyconfig.get_version(), report_errors=report_errors)
 
         parser.add_option(
             "-v", "--verbose", action="count", dest="verbose",
@@ -1705,12 +1866,16 @@ To set a specific wallpaper: %prog /some/local/image.jpg --next""")
                         with self.prepared_lock:
                             logger.info("Adding fetched file %s to used queue immediately after current file" % file)
 
-                            if self.used[self.position] != file and (self.position <= 0 or self.used[self.position - 1] != file):
-                                at_front = self.position == 0
+                            try:
+                                if self.used[self.position] != file and (self.position <= 0 or self.used[self.position - 1] != file):
+                                    at_front = self.position == 0
+                                    self.used.insert(self.position, file)
+                                    self.position += 1
+                                    self.thumbs_manager.mark_active(file=self.used[self.position], position=self.position)
+                                    self.refresh_thumbs_history(file, at_front)
+                            except IndexError:
                                 self.used.insert(self.position, file)
                                 self.position += 1
-                                self.thumbs_manager.mark_active(file=self.used[self.position], position=self.position)
-                                self.refresh_thumbs_history(file, at_front)
 
             except Exception:
                 logger.exception("Exception in process_urls")
@@ -1737,7 +1902,7 @@ To set a specific wallpaper: %prog /some/local/image.jpg --next""")
                 logger.warning("get_wallpaper script is missing or not executable: " + script)
 
             if not file:
-                file = self.gsettings.get_string(self.KEY)
+                file = self.gsettings.get_string('picture-uri')
 
             if not file:
                 return None
@@ -1763,9 +1928,8 @@ To set a specific wallpaper: %prog /some/local/image.jpg --next""")
                 logger.exception("Exception when calling set_wallpaper script")
         else:
             logger.warning("set_wallpaper script is missing or not executable: " + script)
-
-        self.gsettings.set_string(self.KEY, "file://" + wallpaper)
-        self.gsettings.apply()
+            self.gsettings.set_string('picture-uri', "file://" + wallpaper)
+            self.gsettings.apply()
 
     def show_hide_history(self, widget=None):
         if self.thumbs_manager.is_showing("history"):
